/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification;

import java.util.List;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.component.framework.ComponentWsdl;
import org.ow2.petals.component.framework.api.Wsdl;
import org.ow2.petals.component.framework.api.notification.NotificationActor;
import org.ow2.petals.component.framework.util.UtilFactory;

import com.ebmwebsourcing.wsstar.addressing.definition.api.WSAddressingException;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.WSNotificationExtensionFactory;
import com.ebmwebsourcing.wsstar.notification.extension.api.SOAParameterType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;

/**
 * An abstract component notification actor
 * 
 * @author Frederic Gardes
 * 
 */
public class AbstractComponentNotificationActor implements NotificationActor {
    /**
     * The description and the document of the notification actor wsdl managed
     * internally by the component
     */
    private Wsdl wsdl = null;

    protected Logger logger = null;

    /**
     * The constructor
     * 
     * @param wsnWriter
     * @param wsnReader
     * 
     * @throws WSDLException
     * @throws WSNotificationException
     */
    public AbstractComponentNotificationActor(final String wsdlFileName, final Logger logger)
    throws WSDLException, WSNotificationException {
        super();
        this.wsdl = new ComponentWsdl(UtilFactory.getWSDLUtil().createWsdlDescription(
                this.getClass().getResource("/" + wsdlFileName), true));
        this.logger = logger;
    }

    /**
     * Convert a wsdl endpoint to a SOA parameter
     * 
     * @param endpoint
     *            the wsdl endpoint
     * @return The SOA parameter
     * @throws WSNotificationExtensionException
     * @throws WSDLException
     */
    protected SOAParameterType convertToSOAParameter(final Endpoint endpoint)
    throws WSNotificationExtensionException, WSDLException {
        SOAParameterType soaParameter = null;

        soaParameter = WSNotificationExtensionFactory.getInstance().createSOAParameterType();
        soaParameter.setInterface(endpoint.getService().getInterface().getQName());
        soaParameter.setService(endpoint.getService().getQName());
        soaParameter.setEndpoint(endpoint.getName());
        return soaParameter;
    }

    /**
     * Get the SOA parameter into the wsdl
     * 
     * @param qName
     *            The QName matching with the interface of the service into the
     *            wsdl document
     * @return The SOA parameter
     * @throws WSDLException
     * @throws WSNotificationExtensionException
     * @throws WSNotificationException
     * @throws WSAddressingException
     */
    protected SOAParameterType getSOAParameter(final QName qName) throws WSDLException,
    WSNotificationExtensionException {
        final List<Endpoint> endpointList = UtilFactory.getWSDLUtil().getEndpointList(
                this.wsdl.getDescription());
        if (endpointList != null) {
            for (final Endpoint endpoint : endpointList) {
                if (endpoint != null && endpoint.getService() != null
                        && endpoint.getService().getInterface() != null
                        && endpoint.getService().getInterface().getQName() != null) {
                    if (qName.equals(endpoint.getService().getInterface().getQName())) {
                        return this.convertToSOAParameter(endpoint);
                    }
                }
            }
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.notification.NotificationActor
     * #getWsdl()
     */
    public Wsdl getWsdl() {
        return this.wsdl;
    }
}
