/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification.filter.topicexpression;

import java.util.List;

import org.ow2.petals.component.framework.api.Constants;
import org.ow2.petals.component.framework.notification.filter.AbstractFilter;

import com.ebmwebsourcing.wsstar.notification.definition.WSNotificationFactory;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Subscribe;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.TopicExpressionType;
import com.ebmwebsourcing.wsstar.notification.definition.topics.WstopConstants;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.service.fault.WSNotificationFault;
import com.ebmwebsourcing.wsstar.notification.service.topic.WstopTopicManager;

/**
 * A filter on the topic of the subscribe, comparing with the topic of the notif
 * 
 * @author Frederic Gardes
 * 
 */
public class TopicExpressionFilter extends AbstractFilter {

    private TopicExpressionType topicExpressionType = null;

    /**
     * The topic manager
     */
    private WstopTopicManager wstopTopicManager = null;

    public TopicExpressionFilter(final Subscribe subscribe) throws WSNotificationException {
        super(subscribe);
        this.topicExpressionType = WSNotificationFactory.getInstance().createTopicExpressionType();
        this.topicExpressionType.addTopicNameSpace("petals", Constants.WSStar.Topic.NAMESPACE_URI);
        this.topicExpressionType.setDialect(WstopConstants.CONCRETE_TOPIC_EXPRESSION_DIALECT_URI);
        // TODO: add dynamically the topics
        this.wstopTopicManager = new WstopTopicManager(TopicExpressionFilter.class
                .getResourceAsStream(
        "/SupportedTopicsSet.xml"));
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.notification.filter.Filter#isNotifiable
     * (com.ebmwebsourcing.wsstar.notification.definition.base.FilterType,
     * java.lang.Object)
     */
    public boolean isNotifiable(final Object content) {
        String topic = null;

        try {
            if (this.filterType.getTopicExpression() != null) {
                if (content == null || !(content instanceof String)) {
                    return false;
                }
                topic = (String) content;
                // add the subscription to the Topic Manager
                this.wstopTopicManager.storeNewSubscription(this.filterType.getTopicExpression(),
                "1");
                this.topicExpressionType.setContent(topic);
                // TODO create a check with a boolean on response into the
                // topic manager
                final List<String> subscriptionIdList = this.wstopTopicManager
                .getSubscriptionIdsFromTopicsSet(this.topicExpressionType, true);
                this.wstopTopicManager.removeExistingSubscription("1");
                if (subscriptionIdList == null || subscriptionIdList.isEmpty()) {
                    return false;
                }
            }
        } catch (final WSNotificationException e) {
            return false;
        } catch (final WSNotificationFault e) {
            return false;
        } finally {
        }
        return true;
    }
}
