/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification.notify;

import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.xpath.XPathExpressionException;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.api.notification.NotificationBrokerController;
import org.ow2.petals.component.framework.notification.filter.messagecontent.EndpointFilter;
import org.ow2.petals.component.framework.notification.filter.messagecontent.XPathFilter;
import org.ow2.petals.component.framework.notification.filter.topicexpression.TopicExpressionFilter;
import org.ow2.petals.component.framework.notification.subscriptionpolicy.ContextSubscriptionPolicy;
import org.ow2.petals.component.framework.notification.subscriptionpolicy.ProcessSubscriptionPolicy;
import org.ow2.petals.component.framework.notification.subscriptionpolicy.XSLTSubscriptionPolicy;
import org.w3c.dom.Document;

import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Subscribe;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;

/**
 * The abstract notification of a message on a JBI Producer
 * 
 * @author Frederic Gardes
 * 
 */
public abstract class AbstractProducerComponentNotify extends ComponentNotify {

    private TopicExpressionFilter topicExpressionFilter = null;

    private XPathFilter xPathFilter = null;

    private EndpointFilter endpointFilter = null;

    private XSLTSubscriptionPolicy xsltSubscriptionPolicy = null;

    protected ContextSubscriptionPolicy contextSubscriptionPolicy = null;

    private ProcessSubscriptionPolicy processSubscriptionPolicy = null;

    private EndpointReferenceType subscriptionReference = null;

    /**
     * @throws WSNotificationException
     * @throws ParserConfigurationException
     * @throws TransformerException
     * @throws TransformerFactoryConfigurationError
     * @throws WSNotificationExtensionException
     * @throws XPathExpressionException
     * 
     */
    public AbstractProducerComponentNotify(final EndpointReferenceType subscriptionReference,
            final Subscribe subscribe,
            final NotificationBrokerController notificationBrokerController, final Logger logger)
    throws WSNotificationException, WSNotificationExtensionException,
    TransformerFactoryConfigurationError, TransformerException,
    ParserConfigurationException, XPathExpressionException {
        super(notificationBrokerController, logger);
        this.subscriptionReference = subscriptionReference;
        this.topicExpressionFilter = new TopicExpressionFilter(subscribe);
        this.xPathFilter = new XPathFilter(subscribe);
        this.endpointFilter = new EndpointFilter(subscribe);
        if (subscribe.getSubscriptionPolicy() != null) {
            this.xsltSubscriptionPolicy = new XSLTSubscriptionPolicy(subscribe, this.logger);
            this.processSubscriptionPolicy = new ProcessSubscriptionPolicy(subscribe);
            this.contextSubscriptionPolicy = new ContextSubscriptionPolicy(subscribe);
        }
    }

    /**
     * Get the topicExpressionFilter
     * 
     * @return the topicExpressionFilter
     */
    public TopicExpressionFilter getTopicExpressionFilter() {
        return this.topicExpressionFilter;
    }

    /**
     * Process the notify
     * 
     * @param exchange
     *            The exchange
     * @param document
     *            The document to analyze
     * @param status
     *            The status of the notify
     */
    protected void process(final Exchange exchange, Document document) {
        boolean notify = true;

        if (exchange.isProviderRole()) {
            // message content
            notify = this.xPathFilter.isNotifiable(document);
            if (notify) {
                notify = this.endpointFilter.isNotifiable(exchange.getEndpoint());
                if (notify) {
                    // producer properties
                    // TODO create a filter on WS-Resource
                    // apply subscription policies
                    // xslt
                    document = this.xsltSubscriptionPolicy.applyPolicy(document);
                    // context
                    this.contextSubscriptionPolicy.setExchange(exchange);
                    document = this.contextSubscriptionPolicy.applyPolicy(document);
                    // process
                    this.processSubscriptionPolicy.setExchange(exchange);
                    document = this.processSubscriptionPolicy.applyPolicy(document);
                    this.process(document, this.getTopic(), this.subscriptionReference);
                }
            }

        }
    }

    /**
     * Get the topic
     * 
     * @return the topic
     */
    public abstract String getTopic();

    /**
     * Process the notify
     * 
     * @param exchange
     *            The exchange
     * @return an exchange not modified
     * @throws MessagingException
     */
    public abstract Exchange process(Exchange exchange) throws MessagingException;
}
