/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.util;

import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.component.framework.api.util.XSLTUtil;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * This class provides a set of methods to load and transform XML documents. It
 * supports parameter-aware stylesheets (XSLT).
 * 
 * @author Frederic Gardes
 * 
 */
/**
 * @author Frederic Gardes
 * 
 */
public class XSLTUtilImpl implements XSLTUtil {

    private DocumentBuilderFactory dFactory = null;

    private DocumentBuilder builder = null;

    private TransformerFactory transformerFactory = null;

    /**
     * The constructor
     * 
     * @throws ParserConfigurationException
     */
    public XSLTUtilImpl() throws ParserConfigurationException {
        super();
        this.dFactory = DocumentBuilderFactory.newInstance();
        this.dFactory.setNamespaceAware(true);
        this.builder = this.dFactory.newDocumentBuilder();
        // Generate a Transformer.
        this.transformerFactory = TransformerFactory.newInstance();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.XSLTUtil#loadXML(java.lang.String
     * )
     */
    public Document loadXML(final String filename) throws IOException,
    ParserConfigurationException, SAXException {
        return this.builder.parse(new File(filename));
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.XSLTUtil#transformDocument(org
     * .w3c.dom.Document, java.lang.String)
     */
    public Document transformDocument(final Document xmlDocument, final String xsltFilename)
    throws TransformerException, ParserConfigurationException {
        return this.transformDocument(xmlDocument, null, xsltFilename);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.XSLTUtil#transformDocument(org
     * .w3c.dom.Document, javax.xml.transform.Source)
     */
    public Document transformDocument(final Document xmlDocument, final Source xsltSource)
    throws TransformerException, ParserConfigurationException {
        return this.transformDocument(xmlDocument, null, xsltSource);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.XSLTUtil#transformDocumentAsString
     * (org.w3c.dom.Document, java.lang.String)
     */
    public String transformDocumentAsString(final Document xmlDocument, final String xsltFilename)
    throws TransformerFactoryConfigurationError, TransformerException {
        return this.transformDocumentAsString(xmlDocument, null, xsltFilename);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.util.XSLTUtil#
     * transformDocumentAsString(org.w3c.dom.Document,
     * javax.xml.transform.Source)
     */
    public String transformDocumentAsString(final Document xmlDocument, final Source xsltSource)
    throws TransformerFactoryConfigurationError, TransformerException {
        return this.transformDocumentAsString(xmlDocument, null, xsltSource);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.util.XSLTUtil#transformDocument
     * (org.w3c.dom.Document, java.util.Hashtable, java.lang.String)
     */
    public Document transformDocument(final Document xmlDocument,
            final Hashtable<String, String> parameters, final Source xsltSource)
    throws TransformerException, ParserConfigurationException {
        final Transformer transformer = this.createTransformer(parameters, xsltSource);
        return this.transformDocument(xmlDocument, transformer);
    }

    public Document transformDocument(final Document xmlDocument, final Transformer transformer)
    throws TransformerException, ParserConfigurationException {
        Document dstDocument = null;

        if (transformer != null) {
            // Create an empty DOMResult object for the output.
            dstDocument = this.builder.newDocument();
            final DOMResult domResult = new DOMResult(dstDocument);
            // Perform the transformation.
            transformer.transform(this.xslNormalizeAsSource(xmlDocument), domResult);
            transformer.reset();
        }
        return dstDocument;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.util.XSLTUtil#transformDocument
     * (org.w3c.dom.Document, java.util.Hashtable, java.lang.String)
     */
    public Document transformDocument(final Document xmlDocument,
            final Hashtable<String, String> parameters, final String xsltFilename)
    throws TransformerException, ParserConfigurationException {
        final Transformer transformer = this.createTransformer(parameters, new StreamSource(
                xsltFilename));
        return this.transformDocument(xmlDocument, transformer);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.util.XSLTUtil#
     * transformDocumentAsString(org.w3c.dom.Document, java.util.Hashtable,
     * java.lang.String)
     */
    public String transformDocumentAsString(final Document xmlDocument,
            final Hashtable<String, String> parameters, final String xsltFilename)
    throws TransformerFactoryConfigurationError, TransformerException {
        return this.transformDocumentAsString(xmlDocument, parameters, new StreamSource(
                xsltFilename));
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.util.XSLTUtil#
     * transformDocumentAsString(org.w3c.dom.Document, java.util.Hashtable,
     * javax.xml.transform.Source)
     */
    public String transformDocumentAsString(final Document xmlDocument,
            final Hashtable<String, String> parameters, final Source xsltSource)
    throws TransformerFactoryConfigurationError, TransformerException {
        final Transformer transformer = this.createTransformer(parameters, xsltSource);
        final StringWriter stringWriter = new StringWriter();
        final StreamResult streamResult = new StreamResult(stringWriter);
        // Perform the transformation.
        transformer.transform(this.xslNormalizeAsSource(xmlDocument), streamResult);
        transformer.reset();
        // Now you can get the output Node from the DOMResult.
        return stringWriter.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.util.XSLTUtil#createTransformer
     * (java.util.Hashtable, javax.xml.transform.Source)
     */
    public Transformer createTransformer(final Hashtable<String, String> parameters,
            final Source xsltSource) throws TransformerFactoryConfigurationError,
            TransformerException {
        Transformer transformer = null;
        final Source source = this.xslNormalize(xsltSource);
        transformer = this.transformerFactory.newTransformer(source);
        // set transformation parameters
        if (parameters != null) {
            final Enumeration<String> keys = parameters.keys();
            while (keys.hasMoreElements()) {
                final String key = keys.nextElement();
                final String value = parameters.get(key);
                transformer.setParameter(key, value);
            }
        }
        return transformer;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.util.XSLTUtil#createTransformer
     * (javax.xml.transform.Source)
     */
    public Transformer createTransformer(final Source xsltSource)
    throws TransformerFactoryConfigurationError, TransformerException {
        return this.createTransformer(null, xsltSource);
    }

    /**
     * Normalize a document for the xsl engine
     * 
     * @param document
     *            The document to normalize
     * @return a document normalized
     * @throws TransformerException
     */
    private Document xslNormalize(final Document document) throws TransformerException {
        final Transformer transformer = this.transformerFactory.newTransformer();
        final DOMResult dr = new DOMResult();
        final DOMSource source = new DOMSource(document);
        transformer.transform(source, dr);
        transformer.reset();
        return (Document) dr.getNode();
    }

    /**
     * Normalize a document for the xsl engine
     * 
     * @param source
     *            The source to normalize
     * @return a source normalized
     * @throws TransformerException
     */
    private Source xslNormalize(final Source source) throws TransformerException {
        final Transformer transformer = this.transformerFactory.newTransformer();
        final DOMResult dr = new DOMResult();
        transformer.transform(source, dr);
        transformer.reset();
        final Document newDoc = (Document) dr.getNode();
        return new DOMSource(newDoc);
    }

    /**
     * Normalize a document for the xsl engine and return as Source
     * 
     * @param document
     *            The document to normalize
     * @return a source normalized
     * @throws TransformerException
     */
    public Source xslNormalizeAsSource(final Document document) throws TransformerException {
        final Document newDoc = this.xslNormalize(document);
        return new DOMSource(newDoc);
    }
}
